/* iemmatrix-setup autogenerated header-file
 * generated by "./makesource.sh"
 * !! DO NOT MANUALLY EDIT  !!
 */

#ifndef IEMMATRIX_SOURCES_H__
#define IEMMATRIX_SOURCES_H__
void iematrix_setup(void); /* matrix.c */
void iemtx_abs_setup(void); /* mtx_abs.c */
void iemtx_add_setup(void); /* mtx_add.c */
void iemtx_and_setup(void); /* mtx_and.c */
void iemtx_atan_setup(void); /* mtx_atan.c */
void iemtx_bessel_setup(void); /* mtx_bessel.c */
void iemtx_bitand_setup(void); /* mtx_bitand.c */
void iemtx_bitleft_setup(void); /* mtx_bitleft.c */
void iemtx_bitor_setup(void); /* mtx_bitor.c */
void iemtx_bitright_setup(void); /* mtx_bitright.c */
void iemtx_bspline_setup(void); /* mtx_bspline.c */
void iemtx_check_setup(void); /* mtx_check.c */
void iemtx_cholesky_setup(void); /* mtx_cholesky.c */
void iemtx_col_setup(void); /* mtx_col.c */
void iemtx_colon_setup(void); /* mtx_colon.c */
void iemtx_concat_setup(void); /* mtx_concat.c */
void iemtx_conv_setup(void); /* mtx_conv.c */
void iemtx_cos_setup(void); /* mtx_cos.c */
void iemtx_cumprod_setup(void); /* mtx_cumprod.c */
void iemtx_cumsum_setup(void); /* mtx_cumsum.c */
void iemtx_dbtopow_setup(void); /* mtx_dbtopow.c */
void iemtx_dbtorms_setup(void); /* mtx_dbtorms.c */
void iemtx_decay_setup(void); /* mtx_decay.c */
void iemtx_diag_setup(void); /* mtx_diag.c */
void iemtx_diegg_setup(void); /* mtx_diegg.c */
void iemtx_diff_setup(void); /* mtx_diff.c */
void iemtx_dispersive_dline_setup(void); /* mtx_dispersive_dline.c */
void iemtx_distance2_setup(void); /* mtx_distance2.c */
void iemtx_egg_setup(void); /* mtx_egg.c */
void iemtx_eig_setup(void); /* mtx_eig.c */
void iemtx_element_setup(void); /* mtx_element.c */
void iemtx_eq_setup(void); /* mtx_eq.c */
void iemtx_exp_setup(void); /* mtx_exp.c */
void iemtx_eye_setup(void); /* mtx_eye.c */
void iemtx_fft_setup(void); /* mtx_fft.c */
void iemtx_fill_setup(void); /* mtx_fill.c */
void iemtx_find_setup(void); /* mtx_find.c */
void iemtx_gauss_setup(void); /* mtx_gauss.c */
void iemtx_ge_setup(void); /* mtx_ge.c */
void iemtx_gt_setup(void); /* mtx_gt.c */
void iemtx_ifft_setup(void); /* mtx_ifft.c */
void iemtx_index_setup(void); /* mtx_index.c */
void iemtx_int_setup(void); /* mtx_int.c */
void iemtx_inverse_setup(void); /* mtx_inverse.c */
void iemtx_isequal_setup(void); /* mtx_isequal.c */
void iemtx_le_setup(void); /* mtx_le.c */
void iemtx_log_setup(void); /* mtx_log.c */
void iemtx_lt_setup(void); /* mtx_lt.c */
void iemtx_max2_setup(void); /* mtx_max2.c */
void iemtx_mean_setup(void); /* mtx_mean.c */
void iemtx_min2_setup(void); /* mtx_min2.c */
void iemtx_minmax_setup(void); /* mtx_minmax.c */
void iemtx_mul__setup(void); /* mtx_mul~.c */
void iemtx_mul_setup(void); /* mtx_mul.c */
void iemtx_neq_setup(void); /* mtx_neq.c */
void iemtx_not_setup(void); /* mtx_not.c */
void iemtx_ones_setup(void); /* mtx_ones.c */
void iemtx_or_setup(void); /* mtx_or.c */
void iemtx_pack__setup(void); /* mtx_pack~.c */
void iemtx_pivot_setup(void); /* mtx_pivot.c */
void iemtx_pow_setup(void); /* mtx_pow.c */
void iemtx_powtodb_setup(void); /* mtx_powtodb.c */
void iemtx_print_setup(void); /* mtx_print.c */
void iemtx_prod_setup(void); /* mtx_prod.c */
void iemtx_qr_setup(void); /* mtx_qr.c */
void iemtx_rand_setup(void); /* mtx_rand.c */
void iemtx_repmat_setup(void); /* mtx_repmat.c */
void iemtx_resize_setup(void); /* mtx_resize.c */
void iemtx_reverse_setup(void); /* mtx_reverse.c */
void iemtx_rfft_setup(void); /* mtx_rfft.c */
void iemtx_rifft_setup(void); /* mtx_rifft.c */
void iemtx_rmstodb_setup(void); /* mtx_rmstodb.c */
void iemtx_roll_setup(void); /* mtx_roll.c */
void iemtx_row_setup(void); /* mtx_row.c */
void iemtx_scroll_setup(void); /* mtx_scroll.c */
void iemtx_sin_setup(void); /* mtx_sin.c */
void iemtx_size_setup(void); /* mtx_size.c */
void iemtx_slice_setup(void); /* mtx_slice.c */
void iemtx_sndfileread_setup(void); /* mtx_sndfileread.c */
void iemtx_sort_setup(void); /* mtx_sort.c */
void iemtx_spherical_harmonics_setup(void); /* mtx_spherical_harmonics.c */
void iemtx_spherical_radial_setup(void); /* mtx_spherical_radial.c */
void iemtx_sub_setup(void); /* mtx_sub.c */
void iemtx_sum_setup(void); /* mtx_sum.c */
void iemtx_svd_setup(void); /* mtx_svd.c */
void iemtx_trace_setup(void); /* mtx_trace.c */
void iemtx_transpose_setup(void); /* mtx_transpose.c */
void iemtx_unpack__setup(void); /* mtx_unpack~.c */
void iemtx_zeros_setup(void); /* mtx_zeros.c */
#endif /* IEMMATRIX_SOURCES_H__ */

