/* iemmatrix-setup autogenerated setup-file
 * generated by "./makesource.sh"
 * !! DO NOT MANUALLY EDIT  !!
 */

#include "iemmatrix_sources.h"

void iemmatrix_sources_setup(void)
{
	iematrix_setup(); /* matrix.c */
	iemtx_abs_setup(); /* mtx_abs.c */
	iemtx_add_setup(); /* mtx_add.c */
	iemtx_and_setup(); /* mtx_and.c */
	iemtx_atan_setup(); /* mtx_atan.c */
	iemtx_bessel_setup(); /* mtx_bessel.c */
	iemtx_bitand_setup(); /* mtx_bitand.c */
	iemtx_bitleft_setup(); /* mtx_bitleft.c */
	iemtx_bitor_setup(); /* mtx_bitor.c */
	iemtx_bitright_setup(); /* mtx_bitright.c */
	iemtx_bspline_setup(); /* mtx_bspline.c */
	iemtx_check_setup(); /* mtx_check.c */
	iemtx_cholesky_setup(); /* mtx_cholesky.c */
	iemtx_col_setup(); /* mtx_col.c */
	iemtx_colon_setup(); /* mtx_colon.c */
	iemtx_concat_setup(); /* mtx_concat.c */
	iemtx_conv_setup(); /* mtx_conv.c */
	iemtx_cos_setup(); /* mtx_cos.c */
	iemtx_cumprod_setup(); /* mtx_cumprod.c */
	iemtx_cumsum_setup(); /* mtx_cumsum.c */
	iemtx_dbtopow_setup(); /* mtx_dbtopow.c */
	iemtx_dbtorms_setup(); /* mtx_dbtorms.c */
	iemtx_decay_setup(); /* mtx_decay.c */
	iemtx_diag_setup(); /* mtx_diag.c */
	iemtx_diegg_setup(); /* mtx_diegg.c */
	iemtx_diff_setup(); /* mtx_diff.c */
	iemtx_dispersive_dline_setup(); /* mtx_dispersive_dline.c */
	iemtx_distance2_setup(); /* mtx_distance2.c */
	iemtx_egg_setup(); /* mtx_egg.c */
	iemtx_eig_setup(); /* mtx_eig.c */
	iemtx_element_setup(); /* mtx_element.c */
	iemtx_eq_setup(); /* mtx_eq.c */
	iemtx_exp_setup(); /* mtx_exp.c */
	iemtx_eye_setup(); /* mtx_eye.c */
	iemtx_fft_setup(); /* mtx_fft.c */
	iemtx_fill_setup(); /* mtx_fill.c */
	iemtx_find_setup(); /* mtx_find.c */
	iemtx_gauss_setup(); /* mtx_gauss.c */
	iemtx_ge_setup(); /* mtx_ge.c */
	iemtx_gt_setup(); /* mtx_gt.c */
	iemtx_ifft_setup(); /* mtx_ifft.c */
	iemtx_index_setup(); /* mtx_index.c */
	iemtx_int_setup(); /* mtx_int.c */
	iemtx_inverse_setup(); /* mtx_inverse.c */
	iemtx_isequal_setup(); /* mtx_isequal.c */
	iemtx_le_setup(); /* mtx_le.c */
	iemtx_log_setup(); /* mtx_log.c */
	iemtx_lt_setup(); /* mtx_lt.c */
	iemtx_max2_setup(); /* mtx_max2.c */
	iemtx_mean_setup(); /* mtx_mean.c */
	iemtx_min2_setup(); /* mtx_min2.c */
	iemtx_minmax_setup(); /* mtx_minmax.c */
	iemtx_mul__setup(); /* mtx_mul~.c */
	iemtx_mul_setup(); /* mtx_mul.c */
	iemtx_neq_setup(); /* mtx_neq.c */
	iemtx_not_setup(); /* mtx_not.c */
	iemtx_ones_setup(); /* mtx_ones.c */
	iemtx_or_setup(); /* mtx_or.c */
	iemtx_pack__setup(); /* mtx_pack~.c */
	iemtx_pivot_setup(); /* mtx_pivot.c */
	iemtx_pow_setup(); /* mtx_pow.c */
	iemtx_powtodb_setup(); /* mtx_powtodb.c */
	iemtx_print_setup(); /* mtx_print.c */
	iemtx_prod_setup(); /* mtx_prod.c */
	iemtx_qr_setup(); /* mtx_qr.c */
	iemtx_rand_setup(); /* mtx_rand.c */
	iemtx_repmat_setup(); /* mtx_repmat.c */
	iemtx_resize_setup(); /* mtx_resize.c */
	iemtx_reverse_setup(); /* mtx_reverse.c */
	iemtx_rfft_setup(); /* mtx_rfft.c */
	iemtx_rifft_setup(); /* mtx_rifft.c */
	iemtx_rmstodb_setup(); /* mtx_rmstodb.c */
	iemtx_roll_setup(); /* mtx_roll.c */
	iemtx_row_setup(); /* mtx_row.c */
	iemtx_scroll_setup(); /* mtx_scroll.c */
	iemtx_sin_setup(); /* mtx_sin.c */
	iemtx_size_setup(); /* mtx_size.c */
	iemtx_slice_setup(); /* mtx_slice.c */
	iemtx_sndfileread_setup(); /* mtx_sndfileread.c */
	iemtx_sort_setup(); /* mtx_sort.c */
	iemtx_spherical_harmonics_setup(); /* mtx_spherical_harmonics.c */
	iemtx_spherical_radial_setup(); /* mtx_spherical_radial.c */
	iemtx_sub_setup(); /* mtx_sub.c */
	iemtx_sum_setup(); /* mtx_sum.c */
	iemtx_svd_setup(); /* mtx_svd.c */
	iemtx_trace_setup(); /* mtx_trace.c */
	iemtx_transpose_setup(); /* mtx_transpose.c */
	iemtx_unpack__setup(); /* mtx_unpack~.c */
	iemtx_zeros_setup(); /* mtx_zeros.c */
}

